package myTest1;

import java.util.Scanner;

public class IntAryQueue {
	
	private int max;		// 큐 용량		
	private int num;		// 현재 데이터 수
	private int[] que;		// 큐 본체
	
	// 실행 시 예외 : 큐이 비어있음
	public class EmptyIntAryQueueException extends RuntimeException{
		public EmptyIntAryQueueException(){
		}
	}
	
	// 실행 시 예외 : 큐이 가득 참
	public class OverflowIntAryQueueException extends RuntimeException{
		public OverflowIntAryQueueException(){
		}
	}
	
	// 생성자
	public IntAryQueue(int capacity){
		num = 0;
		max = capacity;
		try{
			que = new int[max];			// 큐 본체용 배열을 생성
		}catch(OutOfMemoryError e){	// 생성할 수 없음
			max = 0;
		}
	}

	// 큐에 데이터를 인큐하는 메서드
	public int inQueue(int x) throws OverflowIntAryQueueException{
		if(num >= max){	// 큐이 가득 참
			throw new OverflowIntAryQueueException();
		}
		return que[num++] = x;
	}
	
	// 큐의 맨 첫번째에 있는 데이터를 제거(디큐) 하고 그 값을 반환하는 메서드
	public int deQueue() throws EmptyIntAryQueueException{
		if(num <= 0){
			throw new EmptyIntAryQueueException();
		}
		int data = que[0];
		for(int i=0; i<num-1; i++){ // 0 < 2 // 0 1 2
			que[i] = que[i+1]; // que[0] = que[1] , que[1] = que[2],  que[2] = que[3]
		}
		--num;
		return data;
	}
	
	// 큐의 가장 먼저 빠질 데이터를 몰래 엿보는 메서드(peek 메서드 )
	public int peek() throws EmptyIntAryQueueException{
		if(num <= 0){
			throw new EmptyIntAryQueueException();
		}
		return que[0];
	}
	
	// 큐에서 x를 찾아 인덱스를 반환
	public int indexOf(int x){
		for(int i=num -1; i>=0; i--){
			if(que[i] == x){
				return i;
			}
		}
		return -1;
	}
	
	// 큐을 비움
	public void clear(){
		num = 0;
	}
	
	// 큐의 용량을 반환
	public int capacity(){
		return max;
	}
	
	// 큐에 쌓여있는 데이터 수를 반환
	public int size(){
		return num;
	}
	
	// 큐이 비어있는가?
	public boolean isEmpty(){
		return num <= 0;
	}
	
	// 큐이 가득찼는가?
	public boolean isFull(){
		return num >= max;
	}
	
	// 큐 안의 모든 데이터를 바닥 -> 꼭대기 순서로 출력
	public void dump(){
		if(num <= 0){
			System.out.println("큐이 비어있습니다.");
		}else{
			for(int i=0; i<num; i++){
				System.out.print(que[i] + " ");
			}
			System.out.println();
		}
	}
	
	public static void main(String[] args) {
		Scanner scanner = new Scanner(System.in);
		IntAryQueue queue = new IntAryQueue(64);	// 최대 64개를 푸시할 수 있는 큐
		
		while(true){
			System.out.println("현재 데이터 수 : " + queue.size() + " / " + queue.capacity()); 
			System.out.print("(1) 인큐 (2) 디큐 (3) 피크 (4) 덤프 (0) 종료 : ");
			int menu = scanner.nextInt();
			if(menu == 0) break;
			
			int x;
			switch(menu){
			
			case 1:
				System.out.print("데이터 : ");
				x = scanner.nextInt();
				try{
					queue.inQueue(x);
				}catch(IntAryQueue.OverflowIntAryQueueException e){
					System.out.println("큐가 가득찼습니다.");
				}
				break;
				
			case 2:
				try{
					x = queue.deQueue();
					System.out.println("디큐한 데이터는 " + x + "입니다.");
				}catch(IntAryQueue.EmptyIntAryQueueException e){
					System.out.println("큐가 비어있습니다.");
				}
				break;

			case 3:
				try{
					x = queue.peek();
					System.out.println("피크한 데이터는 " + x + "입니다.");
				}catch(IntAryQueue.EmptyIntAryQueueException e){
					System.out.println("큐가 비어있습니다.");
				}
				break;

			case 4:
				queue.dump();
				break;
			}
		}
	}
}
