package myTest1;

import java.util.Scanner;

public class ShareArrayTwoStack {
	
	private int max;		// 스택 용량		
	private int halfMax;		// 반절 스택 용량		
	private int ptr;		// 스택 포인터1
	private int ptr2;		// 스택 포인터2
	private int[] stk;		// 스택 본체
	
	// 실행 시 예외 : 스택이 비어있음
	public class EmptyIntStackException extends RuntimeException{
		public EmptyIntStackException(){
		}
	}
	
	// 실행 시 예외 : 스택이 가득 참
	public class OverflowIntStackException extends RuntimeException{
		public OverflowIntStackException(){
		}
	}
	
	// 생성자
	public ShareArrayTwoStack(int capacity){
		ptr = 0;
		ptr2 = capacity-1;
		max = capacity;
		halfMax = capacity/2;
		try{
			stk = new int[max];			// 스택 본체용 배열을 생성
		}catch(OutOfMemoryError e){	// 생성할 수 없음
			max = 0;
		}
	}

	// 스택에 데이터를 푸시하는 메서드
	public int oneArrayTwoStackPush(int x) throws OverflowIntStackException{
		// max 값이 몇인지 알아야함.
		// max 5
		// 한번은 배열의 0부터 시작해 넣어야하고
		
		// 또 한번은 배열의 length-1 길이부터 시작에 넣어야함
		if(ptr >= halfMax){	// 스택이 가득 참
			throw new OverflowIntStackException();
		}else{
			stk[ptr++] = x;
		}			
		
		if(ptr2 <= halfMax){
			throw new OverflowIntStackException();
		}else{
			stk[ptr2--] = x;
		}
		
		return x;
	}
	
	// 스택의 꼭대기에 있는 데이터를 제거(팝) 하고 그 값을 반환하는 메서드
	public int pop() throws EmptyIntStackException{
		if(ptr <= 0){
			throw new EmptyIntStackException();
		}
		return stk[--ptr];
	}

	// 스택의 꼭대기에 있는 데이터를 제거(팝) 하고 그 값을 반환하는 메서드
	public int pop2() throws EmptyIntStackException{
		if(ptr2 <= halfMax){
			throw new EmptyIntStackException();
		}
		return stk[--ptr2];
	}
	
	// 스택의 꼭대기에 있는 데이터를 몰래 엿보는 메서드(peek 메서드 )
	public int peek() throws EmptyIntStackException{
		if(ptr <= 0){
			throw new EmptyIntStackException();
		}
		return stk[max-1];
	}
	
	// 스택에서 x를 찾아 인덱스를 반환
	public int indexOf(int x){
		for(int i=ptr -1; i>=0; i--){
			if(stk[i] == x){
				return i;
			}
		}
		return -1;
	}
	
	// 스택을 비움
	public void clear(){
		ptr = 0;
	}
	
	// 스택의 용량을 반환
	public int capacity(){
		return max;
	}
	
	// 스택에 쌓여있는 데이터 수를 반환
	public int size(){
		return ptr;
	}
	
	// 스택이 비어있는가?
	public boolean isEmpty(){
		return ptr <= 0;
	}
	
	// 스택이 가득찼는가?
	public boolean isFull(){
		return ptr >= max;
	}
	
	// 스택 안의 모든 데이터를 바닥 -> 꼭대기 순서로 출력
	public void dump(){
		if(ptr <= 0){
			System.out.println("스택이 비어있습니다.");
		}else{
			for(int i=0; i<max; i++){
				System.out.print(stk[i] + " ");
			}
			System.out.println();
		}
	}
	
	public static void main(String[] args) {
		Scanner scanner = new Scanner(System.in);
		ShareArrayTwoStack s = new ShareArrayTwoStack(64);	// 최대 64개를 푸시할 수 있는 스택
		
		while(true){
			System.out.println("현재 데이터 수 : " + s.size() + " / " + s.capacity()); 
			System.out.println("(1) 푸시 (2) 팝 (3) 피크 (4) 덤프 (5) 종료 : ");
			int menu = scanner.nextInt();
			if(menu == 0) break;
			
			int x;
			switch(menu){
			
			case 1:
				System.out.print("데이터 : ");
				x = scanner.nextInt();
				try{
					s.oneArrayTwoStackPush(x);
				}catch(ShareArrayTwoStack.OverflowIntStackException e){
					System.out.println("스택이 가득찼습니다.");
				}
				break;
				
			case 2:
				try{
					x = s.pop();
//					x = s.pop2();
					System.out.println("팝한 데이터는 " + x + "입니다.");
				}catch(ShareArrayTwoStack.EmptyIntStackException e){
					System.out.println("스택이 비어있습니다.");
				}
				break;

			case 3:
				try{
					x = s.peek();
					System.out.println("피크한 데이터는 " + x + "입니다.");
				}catch(ShareArrayTwoStack.EmptyIntStackException e){
					System.out.println("스택이 비어있습니다.");
				}
				break;

			case 4:
				s.dump();
				break;
			}
		}
	}
}
